package gl

/*
#include "gl.h"
*/
import "C"

func TexParameterf(target, pname Enum, param float32) {
	C.glTexParameterf(C.GLenum(target), C.GLenum(pname), C.GLfloat(param))
}

func TexParameteri(target, pname Enum, param int) {
	C.glTexParameteri(C.GLenum(target), C.GLenum(pname), C.GLint(param))
}

func ActiveTexture(texture Enum) {
	C.glActiveTexture(C.GLenum(texture))
}

func CompressedTexImage2D(target Enum, level int, internalFormat Enum, width, height, border int, imageSize int, data interface{}) {
	p, _ := getptr(data)
	C.glCompressedTexImage2D(C.GLenum(target), C.GLint(level), C.GLenum(internalFormat), C.GLsizei(width), C.GLsizei(height),
		C.GLint(border), C.GLsizei(imageSize), p)
}

func TexImage2D(target Enum, level, internalFormat, width, height, border int, format, typ Enum, data interface{}) {
	p, _ := getptr(data)
	C.glTexImage2D(C.GLenum(target), C.GLint(level), C.GLint(internalFormat), C.GLsizei(width), C.GLsizei(height),
		C.GLint(border), C.GLenum(format), C.GLenum(typ), p)
}

func TexImage3D(target Enum, level, internalFormat, width, height, depth, border int, format, typ Enum, data interface{}) {
	p, _ := getptr(data)
	C.glTexImage3D(C.GLenum(target), C.GLint(level), C.GLint(internalFormat), C.GLsizei(width), C.GLsizei(height), C.GLsizei(depth),
		C.GLint(border), C.GLenum(format), C.GLenum(typ), p)
}

func TexSubImage2D(target Enum, level, xoffset, yoffset, width, height int, format, typ Enum, data interface{}) {
	p, _ := getptr(data)
	C.glTexSubImage2D(C.GLenum(target), C.GLint(level), C.GLint(xoffset), C.GLint(yoffset), C.GLsizei(width), C.GLsizei(height),
		C.GLenum(format), C.GLenum(typ), p)
}

func GenTextures(n int) []Texture {
	t := make([]C.GLuint, n)
	C.glGenTextures(C.GLsizei(n), &t[0])
	tex := make([]Texture, n)
	for i := range tex {
		tex[i] = Texture(t[i])
	}
	return tex
}

func GenTexture() Texture {
	var t C.GLuint
	C.glGenTextures(1, &t)
	return Texture(t)
}

func DeleteTextures(texture []Texture) {
	t := make([]C.GLuint, len(texture))
	for i := range texture {
		t[i] = C.GLuint(texture[i])
	}
	C.glDeleteTextures(C.GLsizei(len(texture)), &t[0])
}

func (t Texture) Bind(target Enum) {
	C.glBindTexture(C.GLenum(target), C.GLuint(t))
}

func (t Texture) Delete() {
	tx := C.GLuint(t)
	C.glDeleteTextures(1, &tx)
}
