package gl

import (
	"errors"
)

func makeShader(typ Enum, src string) (Shader, error) {
	s := CreateShader(typ)
	s.Source(src)
	s.Compile()
	ok := s.IV(COMPILE_STATUS)
	if ok == 0 {
		defer s.Delete()
		return Shader(0), errors.New(s.InfoLog())
	}
	return s, nil
}

func MakeProgram(vs, fs, gs string) (p Program, err error) {
	var v, f, g Shader

	p = CreateProgram()
	defer func() {
		if err != nil {
			p.Delete()
		}
	}()

	v, err = makeShader(VERTEX_SHADER, vs)
	if err != nil {
		return
	}
	defer v.Delete()

	f, err = makeShader(FRAGMENT_SHADER, fs)
	if err != nil {
		return
	}
	defer f.Delete()

	if gs != "" {
		g, err = makeShader(GEOMETRY_SHADER, gs)
		if err != nil {
			return
		}
		defer g.Delete()
		p.Attach(g)
	}

	p.Attach(v)
	p.Attach(f)
	p.Link()
	ok := p.IV(LINK_STATUS)
	if ok == 0 {
		err = errors.New(p.InfoLog())
	}

	return
}
